package net.afyre.haha.demo.solver.smt;

import java.util.Arrays;

/**
 * A logger which sends all messages to 
 * some other loggers.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class SmtCompositeLogger implements SmtLogger {
  /** Loggers which will receive the messages. */
  private final SmtLogger[] loggers;
  
  /**
   * Constructor.
   * @param loggers Loggers which will receive the messages.
   */
  public SmtCompositeLogger(final SmtLogger... loggers) {
    this.loggers = Arrays.copyOf(loggers, loggers.length);
  }
  
  /** {@inheritDoc} */
  @Override
  public void error(final String msg) { 
    for (final SmtLogger logger : loggers) {
      logger.error(msg);
    }
  }

  /** {@inheritDoc} */
  @Override
  public void error(final String msg, 
                    final Throwable cause) { 
    for (final SmtLogger logger : loggers) {
      logger.error(msg, cause);
    }
  }

  /** {@inheritDoc} */
  @Override
  public void model(final SmtModel model) { 
    for (final SmtLogger logger : loggers) {
      logger.model(model);
    }
  }
  
  /** {@inheritDoc} */
  @Override
  public void warning(final String msg) { 
    for (final SmtLogger logger : loggers) {
      logger.warning(msg);
    }
  }

  /** {@inheritDoc} */
  @Override
  public void warning(final String msg, 
                      final Throwable cause) { 
    for (final SmtLogger logger : loggers) {
      logger.warning(msg, cause);
    }
  }

  /** {@inheritDoc} */
  @Override
  public void info(final String msg) { 
    for (final SmtLogger logger : loggers) {
      logger.info(msg);
    }
  }
  
  /** {@inheritDoc} */
  @Override
  public void input(final String text) { 
    for (final SmtLogger logger : loggers) {
      logger.input(text);
    }
  }
  
  /** {@inheritDoc} */
  @Override
  public void output(final String text) {
    for (final SmtLogger logger : loggers) {
      logger.output(text);
    }
  }
}
