package net.afyre.haha.demo.solver.smt;

/**
 * Z3 solver.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public class Z3 extends SmtSolver {
  /** Command line arguments. */
  private static final String[] CMD_ARGS = new String[] {
    "-smt2", // Use SMT2 input language 
    "-in" // read from standard input
  };
  
  /**
   * Constructor.
   * @param solverPath Path to the Z3 executable.
   * @param logger Object used to print log messages.
   */
  protected Z3(final String solverPath,
               final SmtLogger logger) {
    super(solverPath, CMD_ARGS, logger);
  }
  
  /**
   * Create a solver.
   * @param solverPath Path to the Z3 executable.
   * @param logger Object used to print log messages.
   * @return Solver instance.
   */
  public static Z3 getInstance(final String solverPath,
                               final SmtLogger logger) {
    return new Z3(solverPath, logger);
  }
  
  /** {@inheritDoc} */
  @Override
  protected void sendPreamble() throws SmtSolver.CancellationException {
    sendOption("produce-models", "true");
    sendOption("produce-proofs", "false");
    sendOption("soft-timeout", "10000");
    sendOption("mbqi", "true");
  }
}
