package net.afyre.haha.demo.solver.context;

import net.afyre.haha.demo.solver.expr.VariableBinding;
import net.afyre.haha.demo.solver.expr.formulae.Formula;
import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.types.ValueType;

/**
 * Interface of objects used to decompose symbol declarations.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public interface DeclarationVisitor {
  /**
   * Visit an axiom declaration.
   * @param name Axiom name.
   * @param axiom Asserted formula.
   */
  void visitAxiom(String name, Formula axiom);
  
  /**
   * Visit a free variable declaration.
   * @param name Variable name.
   * @param type Variable type.
   */
  void visitTermDeclaration(String name, ValueType type);
   
  /**
   * Visit a value definition.
   * @param name Variable name.
   * @param type Variable name.
   * @param value Value.
   */
  void visitTermDefinition(String name, ValueType type, Term value);
 
  /**
   * Visit a free propositional variable declaration.
   * @param name Variable name.
   */
  void visitPropDeclaration(String name);
  
  /**
   * Visit a propositional constant definition.
   * @param name Constant name.
   * @param value Constant value.
   */
  void visitPropDefinition(String name, Formula value);

  /**
   * Visit a function declaration (without definition).
   * @param name Function name.
   * @param args Argument names and types.
   * @param result Result type.
   */
  void visitFunctionDeclaration(String name,
                                Iterable<VariableBinding> args, 
                                ValueType result);
  
  /**
   * Visit a function definition.
   * @param name Function name.
   * @param args Argument names and types.
   * @param result Result type.
   * @param body Function body.
   */
  void visitFunctionDefinition(String name,
                               Iterable<VariableBinding> args, 
                               ValueType result, 
                               Term body);

  /**
   * Visit a predicate declaration (without definition).
   * @param name Predicate name.
   * @param args Argument names and types.
   */
  void visitPredicateDeclaration(String name,
                                 Iterable<VariableBinding> args);
  
  /**
   * Visit a predicate definition.
   * @param name Predicate name.
   * @param args Argument names and types.
   * @param body Predicate body.
   */
  void visitPredicateDefinition(String name,
                                Iterable<VariableBinding> args, 
                                Formula body);
}
