package net.afyre.haha.demo.solver.context;

import java.util.ArrayList;
import java.util.List;

import net.afyre.haha.demo.solver.expr.VariableBinding;
import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.types.ValueType;

/**
 * Declaration or definition of a function.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class FunctionDeclaration extends AbstractDeclaration {
  /** List of argument names and types */
  private final List<VariableBinding> args;
  
  /** Result type. */
  private final ValueType result;
  
  /** Function body (null in case of a declaration). */
  private final Term body;

  /**
   * Create a function declaration.
   * @param name Function name.
   * @param args Names and types of the arguments.
   * @param result Result type.
   */
  public FunctionDeclaration(final String name, 
                             final Iterable<VariableBinding> args,
                             final ValueType result) {
    super(name);
    this.args = new ArrayList<VariableBinding>();
    for (final VariableBinding arg : args) {
      this.args.add(arg);
    }
    this.result = result;
    this.body = null;
  }
  
  /**
   * Create a function definition.
   * @param name Function name.
   * @param args Names and types of the arguments.
   * @param result Result type.
   * @param body Function body. This can be set to null
   *             to create a declaration instead of a definition.
   */
  public FunctionDeclaration(final String name, 
                             final Iterable<VariableBinding> args,
                             final ValueType result, 
                             final Term body) {
    super(name);
    this.args = new ArrayList<VariableBinding>();
    for (final VariableBinding arg : args) {
      this.args.add(arg);
    }
    this.result = result;
    this.body = body;
  }

  /** {@inheritDoc} */
  @Override
  public void accept(final DeclarationVisitor visitor) {
    if (body == null) {
      visitor.visitFunctionDeclaration(getName(), args, result);
    } else {
      visitor.visitFunctionDefinition(getName(), args, result, body);
    }
  }
  
}
