package net.afyre.haha.demo.solver.context;

import java.util.ArrayList;
import java.util.List;

import net.afyre.haha.demo.solver.expr.VariableBinding;
import net.afyre.haha.demo.solver.expr.formulae.Formula;

/**
 * Declaration or definition of a predicate.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class PredicateDeclaration extends AbstractDeclaration {
  /** List of argument names and types */
  private final List<VariableBinding> args;
  
  /** Predicate body (null in case of a declaration). */
  private final Formula body;

  /**
   * Create a predicate declaration.
   * @param name Predicate name.
   * @param args Names and types of the arguments.
   */
  public PredicateDeclaration(final String name, 
                              final Iterable<VariableBinding> args) {
    super(name);
    this.args = new ArrayList<VariableBinding>();
    for (final VariableBinding arg : args) {
      this.args.add(arg);
    }
    this.body = null;
  }
  
  /**
   * Create a predicate definition.
   * @param name Predicate name.
   * @param args Names and types of the arguments.
   * @param body Predicate body. This can be set to null
   *             to create a declaration instead of a definition.
   */
  public PredicateDeclaration(final String name, 
                              final Iterable<VariableBinding> args,
                              final Formula body) {
    super(name);
    this.args = new ArrayList<VariableBinding>();
    for (final VariableBinding arg : args) {
      this.args.add(arg);
    }
    this.body = body;
  }

  /** {@inheritDoc} */
  @Override
  public void accept(final DeclarationVisitor visitor) {
    if (body == null) {
      visitor.visitPredicateDeclaration(getName(), args);
    } else {
      visitor.visitPredicateDefinition(getName(), args, body);
    }
  }
}
