package net.afyre.haha.demo.solver.expr;

import net.afyre.haha.demo.solver.expr.types.ValueType;
import net.afyre.haha.demo.util.CmpUtil;
import net.afyre.haha.demo.util.HashUtil;

/**
 * Name and type of a variable.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class VariableBinding {
  /** Hash utlilities. */
  private static final HashUtil hashUtil = HashUtil.getInstance();
  
  /** Comparison utlilities. */
  private static final CmpUtil cmpUtil = CmpUtil.getInstance();
  
  /** Name. */
  private final String name;
  
  /** Type. */
  private final ValueType type;
  
  /**
   * Constructor.
   * @param name Name.
   * @param type Type.
   * @throws NullPointerException If an argument is null.
   */
  public VariableBinding(final String name, 
                         final ValueType type) {
    if (name == null) {
      throw new NullPointerException("name");
    }
    if (type == null) {
      throw new NullPointerException("type");
    }
    this.name = name;
    this.type = type;
  }

  /**
   * Get the variable name.
   * @return Name.
   */
  public String getName() {
    return name;
  }
  
  /**
   * Get the variable type.
   * @return Type.
   */
  public ValueType getType() {
    return type;
  }
  
  /** {@inheritDoc} */
  @Override
  public int hashCode() {
    return hashUtil.hash(name, type);
  }
  
  /** {@inheritDoc} */
  @Override
  public boolean equals(final Object obj) {
    if (obj == null) { return false; }
    if (this.getClass().equals(obj.getClass())) {
      final VariableBinding other = (VariableBinding)obj;
      return cmpUtil.safeEquals(name, other.name) &&
             cmpUtil.safeEquals(type,  other.type);
    } else {
      return false;
    }
  }
}
