package net.afyre.haha.demo.solver.expr.formulae.impl;

import java.util.Arrays;

import net.afyre.haha.demo.solver.expr.formulae.FormulaSwitch;
import net.afyre.haha.demo.solver.expr.terms.Term;

/**
 * Predicate call.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class PredicateCall extends AbstractFormula {
  /** Predicate name. */
  private final String predicate;
  
  /** Arguments. */
  private final Term[] arguments;
  
  /**
   * Constructor.
   * @param predicate Predicate name.
   * @param arguments Arguments.
   */
  public PredicateCall(final String predicate, final Term[] arguments) {
    this.predicate = predicate;
    this.arguments = Arrays.copyOf(arguments, arguments.length);
  }

  /**
   * Get the predicate name.
   * @return Predicate name.
   */
  public String getPredicate() {
    return predicate;
  }

  /**
   * Get the arguments.
   * @return Arguments
   */
  public Term[] getArguments() {
    return Arrays.copyOf(arguments, arguments.length);
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final FormulaSwitch<? extends Result> v) {
    return v.casePredicateCall(predicate, arguments);
  }
}
