package net.afyre.haha.demo.solver.expr.impl;

import java.io.StringWriter;

import net.afyre.haha.demo.solver.expr.Expression;
import net.afyre.haha.demo.solver.expr.ExpressionSwitch;
import net.afyre.haha.demo.solver.expr.util.ExprPrinter;
import net.afyre.haha.demo.solver.expr.util.ExprToArraySwitch;
import net.afyre.haha.demo.util.CmpUtil;
import net.afyre.haha.demo.util.HashUtil;

/**
 * Abstract superclass of expressions.
 * 
 * Provides methods common to all expression types.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public abstract class AbstractExpression implements Expression {
  /** Hash utilities. */
  private static final HashUtil hashUtil = HashUtil.getInstance();
  
  /** Comparison utilities. */
  private static final CmpUtil cmpUtil = CmpUtil.getInstance();

  /** Switch used to unpack terms into arrays. */
  private static final ExprToArraySwitch exprToArraySwitch = 
      ExprToArraySwitch.getInstance();
  
  /** Hash code cached after first call. */
  private int hash = -1;
  
  /** {@inheritDoc} */
  @Override
  public String toString() {
    final StringWriter writer = new StringWriter();
    final ExpressionSwitch<Void> printer = new ExprPrinter(writer);
    accept(printer);
    return writer.toString();
  }
  
  /** {@inheritDoc} */
  @Override
  public int hashCode() {
    if (hash == -1) {
      hash = hashUtil.hash(accept(exprToArraySwitch));
    }
    return hash;
  }
  
  /** {@inheritDoc} */
  @Override
  public boolean equals(final Object other) {
    if (other instanceof Expression) {
      if (hashCode() == other.hashCode()) {
        final Object[] firstArray = accept(exprToArraySwitch);
        final Object[] secondArray = 
            ((Expression)other).accept(exprToArraySwitch);
        return cmpUtil.safeEquals(firstArray, secondArray);
      }
    }
    return false;
  }
}
