package net.afyre.haha.demo.solver.expr.terms.impl;

import net.afyre.haha.demo.solver.expr.formulae.Formula;
import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.terms.TermSwitch;

/**
 * Local propositional variable binding.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class LetProp extends AbstractTerm {
  /** Variable name. */
  private final String var;
  
  /** Value expression. */
  private final Formula formula;
  
  /** Expression body. */
  private final Term body;

  /**
   * Constructor.
   * @param var Variable name.
   * @param formula Assigned value.
   * @param body Expression body.
   */
  public LetProp(final String var, 
                 final Formula formula, 
                 final Term body) {
    this.var = var;
    this.formula = formula;
    this.body = body;
  }

  /**
   * Get the assigned variable.
   * @return Variable.
   */
  public String getVar() {
    return var;
  }

  /**
   * Get the assigned value.
   * @return Value.
   */
  public Formula getFormula() {
    return formula;
  }

  /**
   * Get the expression body.
   * @return Expression.
   */
  public Term getBody() {
    return body;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final TermSwitch<? extends Result> v) {
    return v.caseLetProp(var, formula, body);
  }
  
  
}
