package net.afyre.haha.demo.solver.expr.terms.impl;

import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.terms.TermSwitch;

/**
 * Repeat vector.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public final class Repeat extends AbstractTerm {
  /** Source vector. */
  private final Term vector;
  
  /** Number of copies. */
  private final int count;

  /**
   * Constructor.
   * @param vector Source vector.
   * @param count Number of copies.
   */
  public Repeat(final Term vector, final int count) {
    this.vector = vector;
    this.count = count;
  }

  /**
   * Get the source vector.
   * @return Vector.
   */
  public Term getVector() {
    return vector;
  }

  /**
   * Get the number of copies to concatenate.
   * @return Repetition count.
   */
  public int getCount() {
    return count;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final TermSwitch<? extends Result> v) {
    return v.caseRepeat(vector, count);
  }
  
}
