package net.afyre.haha.demo.solver.expr.terms.impl;

import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.terms.TermSwitch;

/**
 * Bit vector selection.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class SelectBits extends AbstractTerm {
  /** Vector. */
  private final Term vector;
  
  /** Leftmost bit index. */
  private final int left;
  
  /** Rightmost bit index. */
  private final int right;

  /**
   * Constructor.
   * @param vector Source vector.
   * @param left Leftmost bit index.
   * @param right Rightmost bit index.
   */
  public SelectBits(final Term vector, 
                    final int left, 
                    final int right) {
    this.vector = vector;
    this.left = left;
    this.right = right;
  }

  /**
   * Get the source vector.
   * @return Vector.
   */
  public Term getVector() {
    return vector;
  }

  /**
   * Get the leftmost bit index.
   * @return Leftmost bit index.
   */
  public int getLeft() {
    return left;
  }

  /**
   * Get the rightmost bit index.
   * @return Rightmost bit index.
   */
  public int getRight() {
    return right;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final TermSwitch<? extends Result> v) {
    return v.caseSelectBits(vector, left, right);
  }
}
