package net.afyre.haha.demo.solver.expr.types;

/**
 * Interface of objects used to create non-value types.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */

public interface NonValueTypeFactory {
  /**
   * Create a non-value type of kind 'Prop'
   * @see NonValueTypeSwitch#caseProp
   * @return Expression object.
   */
  NonValueType createProp();

  /**
   * Create a non-value type of kind 'Function'
   * @see NonValueTypeSwitch#caseFunction
   * @param resultType {@link NonValueTypeSwitch#caseFunction}
   * @param argTypes {@link NonValueTypeSwitch#caseFunction}
   * @return Expression object.
   */
  NonValueType createFunction(ValueType resultType,
                              ValueType... argTypes);

  /**
   * Create a non-value type of kind 'Predicate'
   * @see NonValueTypeSwitch#casePredicate
   * @param argTypes {@link NonValueTypeSwitch#casePredicate}
   * @return Expression object.
   */
  NonValueType createPredicate(ValueType... argTypes);
}
