package net.afyre.haha.demo.ui.annotations;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.source.Annotation;

import net.afyre.haha.demo.vcgen.VCContext;
import net.afyre.haha.demo.vcgen.VerificationCondition;

/**
 * Annotation associated with a verification condition.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public abstract class ConditionAnnotation extends Annotation {
  /** The associated condition. */
  private final VerificationCondition condition;
  
  /**
   * Constructor.
   * @param condition The associated condition
   */
  public ConditionAnnotation(final String id,
                             final VerificationCondition condition) {
    super(id, false, "");
    this.condition = condition;
  }
  
  public VerificationCondition getCondition() {
    return condition;
  }
  
  /**
   * Create textual description of the verification condition.
   * @return Text.
   */
  protected String getVCDescription() {
    final List<String> contexts = new ArrayList<String>();
    
    VCContext current = condition.getContext();
    while (current != null) {
      contexts.add(current.getComment() + "\n");
      current = current.getParent();
    }
    final StringBuilder result = new StringBuilder();
    int indent = 0;
    for (int i = contexts.size() - 1; i >= 0; i--) {
      for (int j = 0; j < indent; j++) {
        result.append(' ');
      }
      indent += 1;
      result.append(contexts.get(i));
    }
    return result.toString();
  }
}
