package net.afyre.haha.demo.ui.handler;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.afyre.haha.demo.haha.Program;
import net.afyre.haha.demo.solver.dummy.VerboseSolver;
import net.afyre.haha.demo.solver.smt.SmtCompositeLogger;
import net.afyre.haha.demo.solver.smt.SmtLogger;
import net.afyre.haha.demo.solver.smt.SmtModel;
import net.afyre.haha.demo.solver.smt.SmtPluginLogger;
import net.afyre.haha.demo.solver.smt.SmtSolver;
import net.afyre.haha.demo.solver.smt.SmtWriterLogger;
import net.afyre.haha.demo.solver.smt.Z3;
import net.afyre.haha.demo.ui.HahaSourceProvider;
import net.afyre.haha.demo.ui.annotations.InvalidAssertAnnotation;
import net.afyre.haha.demo.ui.annotations.UnknownAssertAnnotation;
import net.afyre.haha.demo.ui.prefs.SolverPreferencePage;
import net.afyre.haha.demo.vcgen.HoareVCGen;
import net.afyre.haha.demo.vcgen.VCCollector;
import net.afyre.haha.demo.vcgen.VCContext;
import net.afyre.haha.demo.vcgen.VCPrinter;
import net.afyre.haha.demo.vcgen.VCSolver;
import net.afyre.haha.demo.vcgen.VerificationCondition;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.IAnnotationModelExtension;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleConstants;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.IConsoleView;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.services.ISourceProviderService;
import org.eclipse.xtext.nodemodel.INode;
import org.eclipse.xtext.nodemodel.util.NodeModelUtils;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.ui.editor.XtextEditor;
import org.eclipse.xtext.ui.editor.model.IXtextDocument;
import org.eclipse.xtext.util.concurrent.IUnitOfWork;

/**
 * ??
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class VCGenHandler extends AbstractHandler implements IHandler {
  private static final String CONSOLE_NAME = "VerificationConditions"; 
  
  /** {@inheritDoc} */
  @Override
  public Object execute(final ExecutionEvent event) throws ExecutionException {
    final ISourceProviderService sourceProviderService = 
        (ISourceProviderService) 
          HandlerUtil.getActiveWorkbenchWindow(event).getService(
            ISourceProviderService.class);
    final HahaSourceProvider sourceProvider = 
      (HahaSourceProvider)
        sourceProviderService.getSourceProvider(HahaSourceProvider.VAR_RUNNING);
    
    final Object editorObject =  
      HandlerUtil.getVariable(event, "activeEditor");
    if (! (editorObject instanceof XtextEditor)) {
      MessageBox box = new MessageBox(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell());
      box.setMessage("Oops: " + editorObject.getClass().getCanonicalName());
      box.open(); 
      return null;
    }
    final XtextEditor editor = (XtextEditor)editorObject;
    final IXtextDocument doc = editor.getDocument();
    final IResource editorResource = editor.getResource();
    final IAnnotationModel annotationModel = 
        editor.getDocumentProvider().getAnnotationModel(editor.getEditorInput());
    final Thread thread = new Thread() {
      @Override
      public void run() {
        doc.readOnly(new IUnitOfWork<Object, XtextResource>() {
          public Object exec(XtextResource resource) {
            sourceProvider.setRunning(true);
            try {
              Program prog = (Program) resource.getContents().get(0);
              runVCGen(editorResource, annotationModel, prog, HandlerUtil
                  .getActiveWorkbenchWindow(event).getActivePage());
            } finally {
              sourceProvider.setRunning(false);
            }
            return null;
          }
        });
      }
    };
    sourceProvider.setThread(thread);
    thread.start();
    return null;
  }

  private void print(Writer w, String msg) {
    final String message = msg == null ? "Null" : msg;
    try {
      w.write(message);
      w.flush();
    } catch (IOException e) {
      // Ignore.
    }
  }
  
  private void runVCGen(IResource resource,
                        IAnnotationModel annotationModel,
                        Program program, 
                        IWorkbenchPage page) {
    final HoareVCGen vcgen = new HoareVCGen();
    
    final MessageConsole console = findConsole(CONSOLE_NAME);
    final MessageConsoleStream out = 
        console.newMessageStream();
    final Writer writer = new OutputStreamWriter(out);
    
    final VCSolver solver;
    final IEclipsePreferences prefs = 
        InstanceScope.INSTANCE.getNode("net.afyre.haha.demo.ui");
    final String solverPath = 
        prefs.get(SolverPreferencePage.PREF_SOLVER_PATH, "z3");
    final VCCollector collector = new VCCollector();
    final SmtLogger logger = new SmtCompositeLogger(
                                new SmtPluginLogger(),
                                new SmtWriterLogger(writer)
                             );
    final SmtSolver sigh = Z3.getInstance(solverPath, logger);
    try {
      solver = 
        new VCSolver(new VerboseSolver(sigh, writer));
    } catch (final Throwable e) {
      e.printStackTrace(new PrintWriter(writer)); 
      try { writer.flush(); } catch (final IOException e2) {}
      showConsole(page, console);
      return;
    }
    final VCPrinter printer = new VCPrinter(writer);
        
    print(writer, "====================================\n");
    print(writer, " Collecting verification conditions\n");
    print(writer, "====================================\n\n");
    try {
      vcgen.generateVerificationConditions(null, program, collector);
    } catch (final Exception e) {
      e.printStackTrace(new PrintWriter(writer)); 
      try { writer.flush(); } catch (final IOException e2) {}
    }
    print(writer, "Verification conditions computed.\n");
    print(writer, "Conditions: ");    
    
    for (final VerificationCondition cond : collector.getConditions()) {
      printer.processVerificationCondition(cond);
    }
    printer.end();
    
    print(writer, "\n\n");

    print(writer, "================\n");
    print(writer, " Running solver\n");
    print(writer, "================\n\n");
  
    int valid = 0;
    int invalid = 0;
    int unknown = 0;
    int errors = 0;
    clearMarkers(resource, annotationModel);
    for (final VerificationCondition cond : collector.getConditions()) {
      if (Thread.interrupted()) {
        print(writer, "Interrupt!\n");
        break;
      }
      try {
        solver.processVerificationCondition(cond);
        switch (solver.getLastAnswer()) {
        case VALID: valid++; break;
        case INVALID: 
          createInvalidMarkers(resource, annotationModel, cond, sigh.getLastModel());
          invalid++; 
          break;
        case UNKNOWN: 
          createUnknownMarkers(resource, annotationModel, cond);
          unknown++; 
          break;
        case ERROR:
          createUnknownMarkers(resource, annotationModel, cond);
          errors++;
          break;
        default: assert(false); break;
        }
      } catch (final Exception e) {
        e.printStackTrace(new PrintWriter(writer)); 
        try { writer.flush(); } catch (final IOException e2) {}
      }
    }
    solver.end();

    print(writer, String.format("Total number of conditions: %d\n", 
                                collector.getConditions().size()));

    print(writer, String.format("Valid: %d\n", valid));
    print(writer, String.format("Invalid: %d\n", invalid));
    print(writer, String.format("Unknown: %d\n", unknown));
    print(writer, String.format("Errors: %d\n", errors));
    final int missing= 
        collector.getConditions().size() - valid - invalid - unknown - errors;
    print(writer, String.format("Missing: %d\n", missing));
    
    try { writer.flush(); } catch (final IOException e) { }
    
    showConsole(page, console);
  }
  
  private void clearMarkers(final IResource resource, 
                            final IAnnotationModel annotationModel) {
    if (resource != null) { 
      try {
        resource.deleteMarkers("net.afyre.haha.demo.ui.hahamarker",
                                                   true, 
                                                   IResource.DEPTH_INFINITE);
      } catch (final CoreException e) {
        e.printStackTrace();
      }
    } else  {
      final List<Annotation> annotations = new ArrayList<Annotation>();
      @SuppressWarnings("unchecked")
      final Iterator<Annotation> it = annotationModel.getAnnotationIterator();
      while (it.hasNext()) {
        final Annotation annotation = it.next();
        if (annotation.getType().startsWith("net.afyre.haha")) {
          annotations.add(annotation);
        }
      }
      if (annotationModel instanceof IAnnotationModelExtension) {
        ((IAnnotationModelExtension)annotationModel).replaceAnnotations(
            annotations.toArray(new Annotation[]{}), null);
      } else {
        for (final Annotation annotation : annotations) {
          annotationModel.removeAnnotation(annotation);
        }
      }
    }
  }
  
  private void createUnknownMarkers(final IResource resource,
                                    final IAnnotationModel annotationModel, 
                                    final VerificationCondition vc) {
    if (resource != null) {
      for (final EObject srcElement : vc.getGoal().getSource()) {
        INode node = NodeModelUtils.findActualNodeFor(srcElement);
        
        try {
          final IMarker marker = 
            resource.createMarker("net.afyre.haha.demo.ui.unprovableassertmarker");
          String msg = "Unable to prove the verification condition.\n";
          msg += getVCDescription(vc);
          marker.setAttribute(IMarker.MESSAGE, msg);
          marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
          marker.setAttribute(IMarker.LINE_NUMBER, node.getStartLine());
          marker.setAttribute(IMarker.CHAR_START, node.getOffset());
          marker.setAttribute(IMarker.CHAR_END, node.getTotalEndOffset());
        } catch (CoreException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    } else {
      for (final EObject srcElement : vc.getGoal().getSource()) {
        final INode node = NodeModelUtils.findActualNodeFor(srcElement);
        final Position position = 
            new Position(node.getOffset(), node.getLength());
        final Annotation annotation = new UnknownAssertAnnotation(vc);
        annotationModel.addAnnotation(annotation, position);
      }
    }
  }
  
  private void createInvalidMarkers(final IResource resource,
                                    final IAnnotationModel annotationModel, 
                                    final VerificationCondition vc, 
                                    final SmtModel model) {
    if (resource != null) { 
      for (final EObject srcElement : vc.getGoal().getSource()) {
        INode node = NodeModelUtils.findActualNodeFor(srcElement);
        
        try {
          final IMarker marker = 
            resource.createMarker("net.afyre.haha.demo.ui.unprovableassertmarker");
          String msg = "Verification condition is not valid\n";
          msg += getVCDescription(vc);
          if (model != null) {
            msg = msg + "\nCounterexample: \n" + model.toString();
          }
          marker.setAttribute(IMarker.MESSAGE, msg);
          marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
          marker.setAttribute(IMarker.LINE_NUMBER, node.getStartLine());
          marker.setAttribute(IMarker.CHAR_START, node.getOffset());
          marker.setAttribute(IMarker.CHAR_END, node.getTotalEndOffset());
        } catch (final CoreException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    } else {
      for (final EObject srcElement : vc.getGoal().getSource()) {
        final INode node = NodeModelUtils.findActualNodeFor(srcElement);
        final Position position = 
            new Position(node.getOffset(), node.getLength());
        final Annotation annotation = 
            new InvalidAssertAnnotation(vc, model);
        annotationModel.addAnnotation(annotation, position);
      }   
    }
  }
  
  private String getVCDescription(final VerificationCondition vc) {
    final List<String> contexts = new ArrayList<String>();
    
    VCContext current = vc.getContext();
    while (current != null) {
      contexts.add(current.getComment() + "\n");
      current = current.getParent();
    }
    final StringBuilder result = new StringBuilder();
    for (int i = contexts.size() - 1; i >= 0; i--) {
      result.append(contexts.get(i));
    }
    return result.toString();
  }
  
  private void showConsole(final IWorkbenchPage page, 
                           final MessageConsole console) {
    final String id = IConsoleConstants.ID_CONSOLE_VIEW;
    final Display display = page.getWorkbenchWindow().getShell().getDisplay();
    
    display.syncExec(new Runnable() {
      @Override
      public void run() {
        IConsoleView view;
        try {  
          view = (IConsoleView) page.showView(id);
          view.display(console);
        } catch (final PartInitException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    });  
  }
  
  private MessageConsole findConsole(String name) {
    ConsolePlugin plugin = ConsolePlugin.getDefault();
    IConsoleManager conMan = plugin.getConsoleManager();
    IConsole[] existing = conMan.getConsoles();
    for (int i = 0; i < existing.length; i++)
       if (name.equals(existing[i].getName()))
          return (MessageConsole) existing[i];
    //no console found, so create a new one
    MessageConsole myConsole = new MessageConsole(name, null);
    conMan.addConsoles(new IConsole[]{myConsole});
    return myConsole;
 }
  
}
