package net.afyre.haha.demo.ui.prefs;

import java.io.File;

import org.eclipse.jface.preference.StringButtonFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;

/**
 * Hacked version of FileFiledEditor.
 * 
 * TODO: Comment, cleanup.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * 
 */
public class ExecutableFieldEditor extends StringButtonFieldEditor {
  private final File[] executablePath;
  private final String[] executableExtensions;

  public ExecutableFieldEditor(final String name, final String labelText,
      final Composite parent) {
    super(name, labelText, parent);

    final String systemPath = System.getenv("PATH");
    if (systemPath == null) {
      executablePath = new File[0];
    } else {
      final String[] pathDirs = systemPath.split(File.pathSeparator);
      executablePath = new File[pathDirs.length];
      for (int i = 0; i < pathDirs.length; i++) {
        executablePath[i] = new File(pathDirs[i]);
      }
    }
    final String systemPathExt = System.getenv("PATHEXT");
    if (systemPathExt == null) {
      executableExtensions = new String[0];
    } else {
      final String[] exts = systemPathExt.split(File.pathSeparator);
      executableExtensions = exts;
    }
  }

  private File findFileOnPath(final String file) {
    for (final File dir : executablePath) {
      for (final String ext : executableExtensions) {
        final File exe = new File(dir, file + ext);
        if (exe.exists()) {
          return exe;
        }
      }
      final File exe = new File(dir, file);
      if (exe.exists()) {
        return exe;
      }
    }

    return new File(file);
  }

  protected String changePressed() {
    File f = new File(getTextControl().getText());
    if (!f.exists()) {
      f = null;
    }
    File d = getFile(f);
    if (d == null) {
      return null;
    }

    return d.getAbsolutePath();
  }

  protected boolean checkState() {

    String msg = null;

    String path = getTextControl().getText();
    if (path != null) {
      path = path.trim();
    } else {
      path = "";//$NON-NLS-1$
    }
    if (path.length() == 0) {
      if (!isEmptyStringAllowed()) {
        msg = getErrorMessage();
      }
    } else {
      File file = new File(path);
      if (file.getParent() == null) {
        file = findFileOnPath(path);
      }
      if (!file.exists()) {
        msg = "File does not exist";
      } else if (!file.canExecute()) {
        msg = "File is not executable.";
      }
    }

    if (msg != null) { // error
      showErrorMessage(msg);
      return false;
    }

    if (doCheckState()) {
      clearErrorMessage();
      return true;
    }
    msg = getErrorMessage();
    if (msg != null) {
      showErrorMessage(msg);
    }
    return false;
  }

  private File getFile(File startingDirectory) {
    final FileDialog dialog = new FileDialog(getShell(), SWT.OPEN | SWT.SHEET);
    if (startingDirectory != null) {
      dialog.setFileName(startingDirectory.getPath());
    }
    if (executableExtensions.length > 0) {
      final String[] extensions = new String[executableExtensions.length];
      for (int i = 0; i < executableExtensions.length; i++) {
        extensions[i] = "*" + executableExtensions[i];
      }
      dialog.setFilterExtensions(extensions);
      dialog.setFilterIndex(getFilterIndex(executableExtensions));
    }
    String file = dialog.open();
    if (file != null) {
      file = file.trim();
      if (file.length() > 0) {
        return new File(file);
      }
    }
    return null;
  }
  
  /**
   * Get index of the initial filter used to select files.
   * 
   * @param extensions List of executable extensions.
   * @return Index of '.exe' or 0.
   */
  private int getFilterIndex(final String[] extensions) {
    for (int i = 0; i < extensions.length; i++) {
      if (extensions[i].equalsIgnoreCase(".exe")) {
        return i;
      }
    }
    return 0;
  }
}
