package net.afyre.haha.demo.ui.vcview;

import net.afyre.haha.demo.haha.Program;
import net.afyre.haha.demo.vcgen.HoareVCGen;
import net.afyre.haha.demo.vcgen.tree.Node;
import net.afyre.haha.demo.vcgen.tree.TreeBuilder;

import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.ui.editor.XtextEditor;
import org.eclipse.xtext.ui.editor.model.IXtextDocument;
import org.eclipse.xtext.ui.editor.model.IXtextModelListener;
import org.eclipse.xtext.util.concurrent.IUnitOfWork;

import com.google.inject.Inject;
import com.google.inject.Singleton;

/**
 * Used to synchronize the verification conditions view
 * with the active editor.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
@Singleton
public class VerificationConditionsSynchronizer 
    implements IPartListener, IXtextModelListener {
  private final HoareVCGen vcgen = new HoareVCGen();
  private IXtextDocument lastDocument;
  
  @Inject
  private VerificationConditionsView view;

  /** {@inheritDoc} */
  @Override
  public void modelChanged(final XtextResource resource) {
    view.setTree(processResource(resource));
  }

  /** {@inheritDoc} */
  @Override
  public void partActivated(final IWorkbenchPart part) {
    updateView(part);
  }

  /** {@inheritDoc} */
  @Override
  public void partBroughtToTop(final IWorkbenchPart part) {
    // TODO Auto-generated method stub
    
  }

  /** {@inheritDoc} */
  @Override
  public void partClosed(final IWorkbenchPart part) {
    // TODO Auto-generated method stub
    
  }

  /** {@inheritDoc} */
  @Override
  public void partDeactivated(final IWorkbenchPart part) {
    // TODO Auto-generated method stub
    
  }

  /** {@inheritDoc} */
  @Override
  public void partOpened(final IWorkbenchPart part) {
    // TODO Auto-generated method stub
    
  }
  
  public void start(final IWorkbenchPartSite site) {
    updateView(site.getPage().getActiveEditor());
    site.getWorkbenchWindow().getPartService().addPartListener(this);
  }

  public void stop(IWorkbenchPartSite site) {
    site.getWorkbenchWindow().getPartService().removePartListener(this);
  }
  
  private void updateView(final IWorkbenchPart part) {
    if (part instanceof XtextEditor) {
      final XtextEditor editor = (XtextEditor) part;
      final IXtextDocument document = editor.getDocument();
      if (document != lastDocument) {
        updateView(document);
        if (lastDocument != null) {
          lastDocument.removeModelListener(this);
        }
        document.addModelListener(this);
      }
      lastDocument = document;
    }
  }
  
  private void updateView(final IXtextDocument document) {
    final Node node = 
        document.readOnly(new IUnitOfWork<Node, XtextResource>() {

      @Override
      public Node exec(final XtextResource state) throws Exception {
        return processResource(state);
      }
    
    });
    view.setTree(node);
  }
  
  private Node processResource(final XtextResource state) {
    final Program prog = (Program) state.getContents().get(0);
    final TreeBuilder builder = new TreeBuilder();
    
    vcgen.generateVerificationConditions(null, prog, builder);
    return builder.finish();
  }
}
