package net.afyre.haha.demo.util.tests;

import static org.junit.Assert.*;

import net.afyre.haha.demo.util.ComparablePair;
import net.afyre.haha.demo.util.Pair;

import org.junit.Test;

/**
 * Tests for the {@link ComparablePair} class.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * @see Pair
 */
public class ComparablePairTest {

  /** GetInstance does not return null. */
  @Test
  public void testInstanceNotNull() {
    ComparablePair<?, ?> pair;
    
    pair = ComparablePair.getInstance(1, 2);
    assertNotNull("Null returned from getInstance()", pair);
    pair = ComparablePair.getInstance(null, 2);
    assertNotNull("Null returned from getInstance()", pair);
    pair = ComparablePair.getInstance(1, null);
    assertNotNull("Null returned from getInstance()", pair);
    pair = ComparablePair.getInstance(null, null);
    assertNotNull("Null returned from getInstance()", pair);
  }

  /** Comparison with null returns 1. */
  @Test
  public void testCompareWithNull() {
    final ComparablePair<?, ?> pair = ComparablePair.getInstance(1, 2);    
    assertEquals("Wrong comaprison result", 1, pair.compareTo(null));
  }
  
  /** Compare (1, 1) with (1, 1) */
  @Test
  public void testCompare11with11() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 1);
    b = ComparablePair.getInstance(1, 1);
   assertEquals("Wrong comparison result", 0, a.compareTo(b));
  }


  /** Compare (1, 1) with (2, 2) */
  @Test
  public void testCompare11with12() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 1);
    b = ComparablePair.getInstance(1, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, 1) with (null, null) */
  @Test
  public void testCompare11with1N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 1);
    b = ComparablePair.getInstance(1, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, 1) with (1, 1) */
  @Test
  public void testCompare11with21() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 1);
    b = ComparablePair.getInstance(2, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, 1) with (2, 2) */
  @Test
  public void testCompare11with22() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 1);
    b = ComparablePair.getInstance(2, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, 1) with (null, null) */
  @Test
  public void testCompare11with2N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 1);
    b = ComparablePair.getInstance(2, null);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, 1) with (1, 1) */
  @Test
  public void testCompare11withN1() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 1);
    b = ComparablePair.getInstance(null, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, 1) with (2, 2) */
  @Test
  public void testCompare11withN2() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 1);
    b = ComparablePair.getInstance(null, 2);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, 1) with (null, null) */
  @Test
  public void testCompare11withNN() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 1);
    b = ComparablePair.getInstance(null, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, 2) with (1, 1) */
  @Test
  public void testCompare12with11() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 2);
    b = ComparablePair.getInstance(1, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, 2) with (2, 2) */
  @Test
  public void testCompare12with12() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 2);
    b = ComparablePair.getInstance(1, 2);
   assertEquals("Wrong comparison result", 0, a.compareTo(b));
  }


  /** Compare (1, 2) with (null, null) */
  @Test
  public void testCompare12with1N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 2);
    b = ComparablePair.getInstance(1, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, 2) with (1, 1) */
  @Test
  public void testCompare12with21() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 2);
    b = ComparablePair.getInstance(2, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, 2) with (2, 2) */
  @Test
  public void testCompare12with22() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 2);
    b = ComparablePair.getInstance(2, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, 2) with (null, null) */
  @Test
  public void testCompare12with2N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 2);
    b = ComparablePair.getInstance(2, null);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, 2) with (1, 1) */
  @Test
  public void testCompare12withN1() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 2);
    b = ComparablePair.getInstance(null, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, 2) with (2, 2) */
  @Test
  public void testCompare12withN2() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 2);
    b = ComparablePair.getInstance(null, 2);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, 2) with (null, null) */
  @Test
  public void testCompare12withNN() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, 2);
    b = ComparablePair.getInstance(null, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, null) with (1, 1) */
  @Test
  public void testCompare1Nwith11() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, null);
    b = ComparablePair.getInstance(1, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, null) with (2, 2) */
  @Test
  public void testCompare1Nwith12() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, null);
    b = ComparablePair.getInstance(1, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, null) with (null, null) */
  @Test
  public void testCompare1Nwith1N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, null);
    b = ComparablePair.getInstance(1, null);
   assertEquals("Wrong comparison result", 0, a.compareTo(b));
  }


  /** Compare (1, null) with (1, 1) */
  @Test
  public void testCompare1Nwith21() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, null);
    b = ComparablePair.getInstance(2, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, null) with (2, 2) */
  @Test
  public void testCompare1Nwith22() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, null);
    b = ComparablePair.getInstance(2, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, null) with (null, null) */
  @Test
  public void testCompare1Nwith2N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, null);
    b = ComparablePair.getInstance(2, null);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (1, null) with (1, 1) */
  @Test
  public void testCompare1NwithN1() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, null);
    b = ComparablePair.getInstance(null, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, null) with (2, 2) */
  @Test
  public void testCompare1NwithN2() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, null);
    b = ComparablePair.getInstance(null, 2);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (1, null) with (null, null) */
  @Test
  public void testCompare1NwithNN() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(1, null);
    b = ComparablePair.getInstance(null, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 1) with (1, 1) */
  @Test
  public void testCompare21with11() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 1);
    b = ComparablePair.getInstance(1, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 1) with (2, 2) */
  @Test
  public void testCompare21with12() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 1);
    b = ComparablePair.getInstance(1, 2);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 1) with (null, null) */
  @Test
  public void testCompare21with1N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 1);
    b = ComparablePair.getInstance(1, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 1) with (1, 1) */
  @Test
  public void testCompare21with21() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 1);
    b = ComparablePair.getInstance(2, 1);
   assertEquals("Wrong comparison result", 0, a.compareTo(b));
  }


  /** Compare (2, 1) with (2, 2) */
  @Test
  public void testCompare21with22() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 1);
    b = ComparablePair.getInstance(2, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (2, 1) with (null, null) */
  @Test
  public void testCompare21with2N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 1);
    b = ComparablePair.getInstance(2, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 1) with (1, 1) */
  @Test
  public void testCompare21withN1() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 1);
    b = ComparablePair.getInstance(null, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 1) with (2, 2) */
  @Test
  public void testCompare21withN2() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 1);
    b = ComparablePair.getInstance(null, 2);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 1) with (null, null) */
  @Test
  public void testCompare21withNN() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 1);
    b = ComparablePair.getInstance(null, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 2) with (1, 1) */
  @Test
  public void testCompare22with11() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 2);
    b = ComparablePair.getInstance(1, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 2) with (2, 2) */
  @Test
  public void testCompare22with12() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 2);
    b = ComparablePair.getInstance(1, 2);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 2) with (null, null) */
  @Test
  public void testCompare22with1N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 2);
    b = ComparablePair.getInstance(1, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 2) with (1, 1) */
  @Test
  public void testCompare22with21() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 2);
    b = ComparablePair.getInstance(2, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 2) with (2, 2) */
  @Test
  public void testCompare22with22() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 2);
    b = ComparablePair.getInstance(2, 2);
   assertEquals("Wrong comparison result", 0, a.compareTo(b));
  }


  /** Compare (2, 2) with (null, null) */
  @Test
  public void testCompare22with2N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 2);
    b = ComparablePair.getInstance(2, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 2) with (1, 1) */
  @Test
  public void testCompare22withN1() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 2);
    b = ComparablePair.getInstance(null, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 2) with (2, 2) */
  @Test
  public void testCompare22withN2() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 2);
    b = ComparablePair.getInstance(null, 2);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, 2) with (null, null) */
  @Test
  public void testCompare22withNN() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, 2);
    b = ComparablePair.getInstance(null, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, null) with (1, 1) */
  @Test
  public void testCompare2Nwith11() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, null);
    b = ComparablePair.getInstance(1, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, null) with (2, 2) */
  @Test
  public void testCompare2Nwith12() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, null);
    b = ComparablePair.getInstance(1, 2);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, null) with (null, null) */
  @Test
  public void testCompare2Nwith1N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, null);
    b = ComparablePair.getInstance(1, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, null) with (1, 1) */
  @Test
  public void testCompare2Nwith21() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, null);
    b = ComparablePair.getInstance(2, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (2, null) with (2, 2) */
  @Test
  public void testCompare2Nwith22() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, null);
    b = ComparablePair.getInstance(2, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (2, null) with (null, null) */
  @Test
  public void testCompare2Nwith2N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, null);
    b = ComparablePair.getInstance(2, null);
   assertEquals("Wrong comparison result", 0, a.compareTo(b));
  }


  /** Compare (2, null) with (1, 1) */
  @Test
  public void testCompare2NwithN1() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, null);
    b = ComparablePair.getInstance(null, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, null) with (2, 2) */
  @Test
  public void testCompare2NwithN2() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, null);
    b = ComparablePair.getInstance(null, 2);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (2, null) with (null, null) */
  @Test
  public void testCompare2NwithNN() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(2, null);
    b = ComparablePair.getInstance(null, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (null, 1) with (1, 1) */
  @Test
  public void testCompareN1with11() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 1);
    b = ComparablePair.getInstance(1, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 1) with (2, 2) */
  @Test
  public void testCompareN1with12() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 1);
    b = ComparablePair.getInstance(1, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 1) with (null, null) */
  @Test
  public void testCompareN1with1N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 1);
    b = ComparablePair.getInstance(1, null);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 1) with (1, 1) */
  @Test
  public void testCompareN1with21() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 1);
    b = ComparablePair.getInstance(2, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 1) with (2, 2) */
  @Test
  public void testCompareN1with22() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 1);
    b = ComparablePair.getInstance(2, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 1) with (null, null) */
  @Test
  public void testCompareN1with2N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 1);
    b = ComparablePair.getInstance(2, null);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 1) with (1, 1) */
  @Test
  public void testCompareN1withN1() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 1);
    b = ComparablePair.getInstance(null, 1);
   assertEquals("Wrong comparison result", 0, a.compareTo(b));
  }


  /** Compare (null, 1) with (2, 2) */
  @Test
  public void testCompareN1withN2() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 1);
    b = ComparablePair.getInstance(null, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 1) with (null, null) */
  @Test
  public void testCompareN1withNN() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 1);
    b = ComparablePair.getInstance(null, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (null, 2) with (1, 1) */
  @Test
  public void testCompareN2with11() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 2);
    b = ComparablePair.getInstance(1, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 2) with (2, 2) */
  @Test
  public void testCompareN2with12() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 2);
    b = ComparablePair.getInstance(1, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 2) with (null, null) */
  @Test
  public void testCompareN2with1N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 2);
    b = ComparablePair.getInstance(1, null);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 2) with (1, 1) */
  @Test
  public void testCompareN2with21() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 2);
    b = ComparablePair.getInstance(2, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 2) with (2, 2) */
  @Test
  public void testCompareN2with22() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 2);
    b = ComparablePair.getInstance(2, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 2) with (null, null) */
  @Test
  public void testCompareN2with2N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 2);
    b = ComparablePair.getInstance(2, null);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, 2) with (1, 1) */
  @Test
  public void testCompareN2withN1() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 2);
    b = ComparablePair.getInstance(null, 1);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (null, 2) with (2, 2) */
  @Test
  public void testCompareN2withN2() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 2);
    b = ComparablePair.getInstance(null, 2);
   assertEquals("Wrong comparison result", 0, a.compareTo(b));
  }


  /** Compare (null, 2) with (null, null) */
  @Test
  public void testCompareN2withNN() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, 2);
    b = ComparablePair.getInstance(null, null);
   assertEquals("Wrong comparison result", 1, a.compareTo(b));
  }


  /** Compare (null, null) with (1, 1) */
  @Test
  public void testCompareNNwith11() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, null);
    b = ComparablePair.getInstance(1, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, null) with (2, 2) */
  @Test
  public void testCompareNNwith12() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, null);
    b = ComparablePair.getInstance(1, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, null) with (null, null) */
  @Test
  public void testCompareNNwith1N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, null);
    b = ComparablePair.getInstance(1, null);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, null) with (1, 1) */
  @Test
  public void testCompareNNwith21() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, null);
    b = ComparablePair.getInstance(2, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, null) with (2, 2) */
  @Test
  public void testCompareNNwith22() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, null);
    b = ComparablePair.getInstance(2, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, null) with (null, null) */
  @Test
  public void testCompareNNwith2N() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, null);
    b = ComparablePair.getInstance(2, null);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, null) with (1, 1) */
  @Test
  public void testCompareNNwithN1() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, null);
    b = ComparablePair.getInstance(null, 1);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, null) with (2, 2) */
  @Test
  public void testCompareNNwithN2() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, null);
    b = ComparablePair.getInstance(null, 2);
   assertEquals("Wrong comparison result", -1, a.compareTo(b));
  }


  /** Compare (null, null) with (null, null) */
  @Test
  public void testCompareNNwithNN() {
    final ComparablePair<Integer, Integer> a, b;
              
    a = ComparablePair.getInstance(null, null);
    b = ComparablePair.getInstance(null, null);
   assertEquals("Wrong comparison result", 0, a.compareTo(b));
  }

  
}
