package net.afyre.haha.demo.util.tests;

import static net.afyre.haha.demo.util.tests.Util.*;
import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.Iterator;

import net.afyre.haha.demo.util.Function;
import net.afyre.haha.demo.util.MappedIterable;
import net.afyre.haha.demo.util.tests.MappedIteratorTest;

import org.junit.Test;


/**
 * Tests for the {@link MappedIterable} class.
 * Test cases defined here are usually duplicated
 * in MappedIteratorTest.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * @see MappedIteratorTest
 */
public class MappedIterableTest {

  /** GetInstance does not return null. */
  @Test
  public void testInstanceNotNull() {
    final Integer[] array = { 1, 2, 3, 4 };
    final Iterable<Integer> source = Arrays.asList(array);
    final Function<Integer, Integer> function = new Inc();
    final Iterable<Integer> iterable = 
        MappedIterable.getInstance(source, function);
    assertNotNull("Null returned from getInstance", iterable);
  }

  /** NPE is thrown for null source. */
  @Test(expected=NullPointerException.class)
  public void testNullSource() {
    MappedIterable.getInstance(null, new Inc());
  }
  
  /** NPE is thrown for null function. */
  @Test(expected=NullPointerException.class)
  public void testNullFunction() {    
    final Integer[] array = { 1, 2, 3, 4 };
    MappedIterable.getInstance(Arrays.asList(array), null);
  }
  
  /** Returned iterator is not null. */
  @Test
  public void testIteratorNotNull() {
    final Integer[] array = { 2, 1, 4, 3 };
    final Iterable<Integer> source = Arrays.asList(array);
    final Function<Integer, Integer> function = new Inc();
    final Iterable<Integer> iterable = 
        MappedIterable.getInstance(source, function);
    final Iterator<Integer> it = iterable.iterator();
    assertNotNull("Null iterator returned", it);
  }
  
  /** Mapping is applied correctly. */
  @Test
  public void testSimpleMap() {
    final Integer[] array = { 2, 1, 4, 3 };
    final Iterable<Integer> source = Arrays.asList(array);
    final Function<Integer, Integer> function = new Inc();
    final Iterable<Integer> iterable = 
        MappedIterable.getInstance(source, function);
    final Iterator<Integer> it = iterable.iterator();
    assertIteratorReturns(it, 3, 2, 5, 4);
  }
  
  /** Iterator works correctly for empty sequences. */
  @Test
  public void testEmpty() {
    final Integer[] array = { };
    final Iterable<Integer> source = Arrays.asList(array);
    final Function<Integer, Integer> function = new Inc();
    final Iterable<Integer> iterable = 
        MappedIterable.getInstance(source, function);
    final Iterator<Integer> it = iterable.iterator();
    assertIteratorReturns(it);
  }

  private static class Inc implements Function<Integer, Integer> {
    /** {@inheritDoc} */
    @Override
    public Integer apply(final Integer arg) {
      return arg + 1;
    }
  }
}
