package net.afyre.haha.demo.util.symboltable;

/**
 * Generic interface of a symbol table frame. 
 * 
 * Frames form a hierarchical structure. When a symbol is looked up 
 * in a frame and it is not found there, parent frame is searched. 
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 * @param <Symbol> Type of symbols (identifiers).
 * @param <SymbolInfo> Type of information associated with a symbol.
 */
public interface Frame<Symbol, SymbolInfo> {
  /**
   * Create an empty child frame.
   * @return New frame.
   */
  Frame<Symbol, SymbolInfo> createChildFrame();
  
  /**
   * Get the parent frame.
   * @return Parent frame or null (for root frames).
   */
  Frame<Symbol, SymbolInfo> getParent();
  
  /**
   * Add new symbol or update symbol information.
   * @param symbol Symbol to be added or updated.
   * @param info New or updated symbol information.
   * @throws NullPointerException if a parameter is null.
   */
  void setSymbolInfo(Symbol symbol, SymbolInfo info);
  
  /**
   * Find information associated with a symbol.
   * @param symbol Symbol to find.
   * @return Symbol information or null (if not found).
   * @throws NullPointerException If the parameter is null.
   */
  SymbolInfo find(Symbol symbol);
}
