package net.afyre.haha.demo.vcgen;

import java.util.HashMap;
import java.util.Map;

import net.afyre.haha.demo.util.Pair;

/**
 * Utility used to generate unique labels.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public class LabelGenerator {
  
  
  private static final class Frame {
    private static final String DEFAULT_CONTEXT = "";
    private final Frame parent;
    
    private final Map<Pair<String, String>, Integer> counters;
    
    public Frame(final Frame parent) {
      this.parent = parent;
      this.counters = new HashMap<Pair<String, String>, Integer>();
    }
    
    public String createUniqueName(final String base) {
      return createUniqueName(DEFAULT_CONTEXT, base);
    }
    
    public String createUniqueName(final String context,
                                   final String base) {
      final int count = getCount(context, base);
      
       counters.put(Pair.getInstance(context, base),  count + 1);
       if (count == 0 && !mayCauseConflict(base)) {
         return base;
       } else {
         return base + "_" + count;
       }
    }
    
    private boolean mayCauseConflict(final String name) {
      int i;
      
      i = name.lastIndexOf('_');
      if (i == -1) { return false; }
      
      while (i < name.length()) {
        final int ch = name.codePointAt(i);
        if (!Character.isDigit(ch)) {
          return false;
        }
        i = i + Character.charCount(ch);
      }
      
      return true;
    }
    
    private int getCount(final String context, 
                         final String base) {
      final Pair<String, String> key = 
          Pair.getInstance(context, base);
      if (counters.containsKey(key)) {
        return counters.get(key);
      } else {
        if (parent != null) {
          return parent.getCount(context, base);
        } else {
          return 0;
        }
      }
    }
    
    public Frame getParent() {
      return parent;
    }
    
  }

  private Frame currentFrame;
  
  public LabelGenerator() {
    currentFrame = new Frame(null);
  }
  
  public void newFrame() {
    currentFrame = new Frame(currentFrame);
  }
  
  public void popFrame() {
    currentFrame = currentFrame.getParent();
  }
  
  public String createUniqueName(final String context,
                                 final String base) {
    return currentFrame.createUniqueName(context, base);
  }

  public String createUniqueName(final String base) {
    return currentFrame.createUniqueName(base);
  }
  
}
