package net.afyre.haha.demo.vcgen;

import java.util.List;

import com.google.common.collect.ImmutableList;

/**
 * Single verification condition.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class VerificationCondition {
  /** Assumptions valid for this condition. */
  private final List<DocumentedFormula> assumptions;
  
  /** Condition expressed as a formula. */
  private final DocumentedFormula goal;

  /** Source of this condition. */
  private final VCContext context;

  /**
   * Constructor.
   * @param formula Condition expressed as a formula.
   * @param name Condition name.
   * @param source Source elements associated with this condition.
   */
  public VerificationCondition(final List<DocumentedFormula> assumptions,
                               final DocumentedFormula goal,
                               final VCContext context) {
    this.assumptions = ImmutableList.copyOf(assumptions);
    this.goal = goal;
    this.context = context;
  }

  /**
   * Return the list of assumptions.
   * Returned list is immutable.
   * @return Assumptions. 
   */
  public List<DocumentedFormula> getAssumptions() {
    return assumptions;
  }

  /**
   * Get the goal formula associated with this condition.
   * @return Goal.
   */
  public DocumentedFormula getGoal() {
    return goal;
  }
  
  /**
   * Get condition context.
   * @return Context.
   */
  public VCContext getContext() {
    return context;
  }
}
